-- Organization type
CREATE TABLE org_type (
   org_type_id   integer not null primary key,
   name          varchar(1024) not null
);

COMMENT on TABLE org_type is 'Organization type - indicating whether it is a company or consumer etc.';
COMMENT ON COLUMN org_type.org_type_id is 'Unique identifier for the organization type';
COMMENT ON COLUMN org_type.name is 'Name of the organization type';

-- Default values for the org_type table
INSERT INTO org_type values (1, 'Company');
INSERT INTO org_type values (2, 'Consumer');
INSERT INTO org_type values (3, 'Threat Exchange');

-- Email type
CREATE TABLE email_type (
   email_type_id integer not null primary key,
   name          varchar(1024) not null
);

COMMENT on TABLE email_type is 'Email type - indicating whether it is a work or personal email';
COMMENT ON COLUMN email_type.email_type_id is 'Unique identifier for the email type';
COMMENT ON COLUMN email_type.name is 'Name of the email type';

-- Default values for the org_type table
INSERT INTO email_type values (1, 'Personal');
INSERT INTO email_type values (2, 'Work');

--
-- Table: CONTACT
--
CREATE TABLE contact (
   contact_id              bigserial not null primary key,
   name                    varchar(1024),
   first_name              varchar(255),
   middle_name             varchar(255),
   last_name               varchar(255),
   title                   varchar(128),
   email_id                varchar(1024) not null unique,
   phone                   varchar(32),
   address_line_1          varchar(255),
   address_line_2          varchar(255),
   city                    varchar(255),
   state                   varchar(255),
   zip_code                varchar(32),
   country                 varchar(255),
   department              varchar(255),
   update_version          integer not null default 0
);

COMMENT on TABLE contact is 'Contact info';
COMMENT ON COLUMN contact.contact_id is 'Unique identifier for the contact';
COMMENT ON COLUMN contact.name is 'Name';
COMMENT ON COLUMN contact.first_name is 'First Name';
COMMENT ON COLUMN contact.middle_name is 'Middle Name';
COMMENT ON COLUMN contact.last_name is 'Last Name';
COMMENT ON COLUMN contact.title is 'Title';
COMMENT ON COLUMN contact.email_id is 'Email id';
COMMENT ON COLUMN contact.phone is 'Phone Number';
COMMENT ON COLUMN contact.address_line_1 is 'Address line 1';
COMMENT ON COLUMN contact.address_line_2 is 'Address line 2';
COMMENT ON COLUMN contact.city is 'City';
COMMENT ON COLUMN contact.state is 'State';
COMMENT ON COLUMN contact.zip_code is 'Zip (postal) code';
COMMENT ON COLUMN contact.country is 'Country';
COMMENT ON COLUMN contact.department is 'Department';
COMMENT ON COLUMN contact.update_version is 'Version counter used to check for concurrent updates';


--
-- Table: organization
--
CREATE TABLE organization (
   org_id                     serial not null primary key,
   org_type_id                integer not null references org_type(org_type_id),
   name                       varchar(1024) not null,
   admin_contact_id           bigint references contact(contact_id),
   billing_contact_id         bigint references contact(contact_id),
   registration_password      varchar(255),
   analytics_profile          varchar(255),
   analytics_version          varchar(255),
   storage_profile            varchar(255),
   create_time                timestamp with time zone  not null,
   last_update_time           timestamp with time zone  ,
   update_version             integer not null default 0,
   mode                       integer DEFAULT 2,
   trial_end_time             timestamp with time zone,
   create_admin               varchar(255),
   paid_start_time            timestamp with time zone
);

alter table organization add column change_timestamps varchar(4096);
COMMENT ON COLUMN organization.change_timestamps is 'Timestamps used to track change events for the org';

COMMENT on TABLE organization is 'Organization - could be company or family group etc.';
COMMENT ON COLUMN organization.org_id is 'Unique identifier for the organization';
COMMENT ON COLUMN organization.org_type_id is 'Unique identifier for the organization type';
COMMENT ON COLUMN organization.name is 'Company Name';
COMMENT ON COLUMN organization.admin_contact_id is 'Admin contact for the org';
COMMENT ON COLUMN organization.billing_contact_id is 'Billing contact for the org';
COMMENT ON COLUMN organization.registration_password is 'Registration password - used for bulk registration';
COMMENT ON COLUMN organization.analytics_profile is 'Analytics Profile id';
COMMENT ON COLUMN organization.analytics_version is 'Analytics Version string';
COMMENT ON COLUMN organization.storage_profile is 'Storage profile string';
COMMENT ON COLUMN organization.update_version is 'Version counter used to check for concurrent updates';
COMMENT ON COLUMN organization.mode IS 'Trial or paid mode';
COMMENT ON COLUMN organization.trial_end_time IS 'End trial time';
COMMENT ON COLUMN organization.create_admin IS 'Admin who created this organization';
COMMENT ON COLUMN organization.paid_start_time IS 'Paid start time';

--
-- Index on registration_password to allow query for automated (SMS) registration
--
create unique index idx_org_reg_pwd on organization(registration_password);


--
-- Table: org_domain -  Organization Domain info
--
create table org_domain (
    org_domain_id      serial        not null primary key,
    org_id             integer       not null references organization(org_id),
    domain_name        varchar(1024) not null unique,
    is_default         boolean not null,
    description        varchar(1024),
    update_version     integer not null default 0
);

COMMENT on TABLE org_domain is 'Association of a domain with an organization';
COMMENT ON COLUMN org_domain.org_domain_id is 'Unique identifier for the org domain definition';
COMMENT ON COLUMN org_domain.org_id is 'Unique identifier for the organization';
COMMENT ON COLUMN org_domain.domain_name is 'Domain name - Assuming this is unique across all companies';
COMMENT ON COLUMN org_domain.is_default is 'Is this the default domain for the organization?';
COMMENT ON COLUMN org_domain.description is 'Description';
COMMENT ON COLUMN org_domain.update_version is 'Version counter used to check for concurrent updates';

--
-- Table: org_config_param -  Configuration params for the organization
--
create table org_config_param (
    org_id             integer       not null references organization(org_id),
    param_id           integer       not null,
    description        varchar(1024),
    param_value        varchar(4096),
    update_version     integer not null default 0,
    primary key (org_id, param_id)
);

COMMENT on TABLE org_config_param is 'Association of a config param with an organization';
COMMENT ON COLUMN org_config_param.org_id is 'Unique identifier for the organization';
COMMENT ON COLUMN org_config_param.param_id is 'Identifier for the parameter.';
COMMENT ON COLUMN org_config_param.description is 'Description';
COMMENT ON COLUMN org_config_param.param_value is 'Value of the parameter represented as a string';
COMMENT ON COLUMN org_config_param.update_version is 'Version counter used to check for concurrent updates';

--
-- Table: device_group:  Group of devices - Could be for monitoring or policy control.
--
CREATE TABLE device_group (
    device_group_id      serial  not null primary key,
    org_id               integer  not null references organization(org_id),
    policy_control       boolean not null,
    is_default           boolean not null,
    priority             integer not null default 10 check (priority between 0 and 100),
    name                 varchar(1024) not null,
    description          varchar(1024),
    analytics_profile    varchar(255),
    analytics_version    varchar(255),
    admin_contact_id     bigint references contact(contact_id),
    policy_info          varchar(4096),
    policy_info_v2       text,
    update_version       integer not null default 0,
    priority_level	     integer not null,
    unique (org_id, name)
);

alter table device_group add column change_timestamps varchar(4096);
COMMENT ON COLUMN device_group.change_timestamps is 'Timestamps used to track change events for the device groudp';

--
-- Index on org_id to allow query by org_id
--
create index idx_dg_org_id on device_group(org_id);

COMMENT on TABLE device_group is 'Group of devices controlled using a single policy';
COMMENT ON COLUMN device_group.device_group_id is 'Unique identifier for the device group';
COMMENT ON COLUMN device_group.org_id is 'Organization id';
COMMENT ON COLUMN device_group.policy_control is 'Boolean indicator whether the device policy is controlled or not';
COMMENT ON COLUMN device_group.is_default is 'Boolean indicator whether this is the default group for the organization';
COMMENT ON COLUMN device_group.priority is 'Priority or importance of this group';
COMMENT ON COLUMN device_group.name is 'Device group name';
COMMENT ON COLUMN device_group.description is 'Device Group Description';
COMMENT ON COLUMN device_group.analytics_profile is 'Analytics profile associated with the device group';
COMMENT ON COLUMN device_group.analytics_version is 'Analytics version associated with the device group';
COMMENT ON COLUMN device_group.admin_contact_id is 'Administrative contact for this device group';
COMMENT ON COLUMN device_group.policy_info is 'Serialized Policy used to control devices associated with this device group';
COMMENT ON COLUMN device_group.update_version is 'Version counter used to check for concurrent updates';


--
-- Table: device_owner
--
CREATE TABLE device_owner (
   device_owner_id     bigserial not null primary key,
   org_id              integer references organization(org_id),
   email_id            varchar(255),
   owner_contact_id    bigint references contact(contact_id),
   activation_code     varchar(16),
   num_devices         integer not null default 0,
   expire_time         timestamp with time zone  not null,
   create_time         timestamp with time zone  not null,
   last_update_time    timestamp with time zone,
   device_group_id     integer references device_group(device_group_id),
   update_version      integer      not null default 0
);

COMMENT ON TABLE device_owner is 'Owner for each device';
COMMENT ON COLUMN device_owner.device_owner_id is 'Unique identifier for the device owner';
COMMENT ON COLUMN device_owner.org_id is 'Organization id';
COMMENT ON COLUMN device_owner.email_id is 'Email id used to register the device with the company';
COMMENT ON COLUMN device_owner.owner_contact_id is 'Reference to contact record with details';
COMMENT ON COLUMN device_owner.activation_code is 'Activation code for this device owner';
COMMENT ON COLUMN device_owner.num_devices is 'Number of devices owned by this person';
COMMENT ON COLUMN device_owner.expire_time is 'When the activation_code expires';
COMMENT ON COLUMN device_owner.create_time is 'When the device_owner entry was created';
COMMENT ON COLUMN device_owner.last_update_time is 'When the device_owner entry was last updated';
COMMENT ON COLUMN device_owner.update_version is 'Version counter used to check for concurrent updates';

-- Index on the activation code
create index deviceowner_code_idx on device_owner(activation_code);
--
-- Add a unique index on org_id, lower(email_id)
--
create unique index device_owner_org_id_email_id_uniq on device_owner (org_id, lower(email_id));

--
-- Device Type
--
CREATE TABLE device_type (
   device_type_id   integer not null primary key,
   name             varchar(1024) not null
);

COMMENT on TABLE device_type is 'Device type - indicating whether it is a Windows, Android, Iphone.';
COMMENT ON COLUMN device_type.device_type_id is 'Unique identifier for the device type';
COMMENT ON COLUMN device_type.name is 'Name of the device type';

--
-- Default values for the device_type table
--
INSERT INTO device_type values (1, 'Windows');
INSERT INTO device_type values (2, 'Android');
INSERT INTO device_type values (3, 'Mac');
INSERT INTO device_type values (4, 'Ios');
INSERT INTO device_type values (10, 'Other');

--
-- Table: device: device_id, device_guid, device_type
--
CREATE TABLE device (
    device_id          serial       not null primary key,
    device_type_id     integer      not null references device_type(device_type_id),
    device_owner_id    integer      references device_owner(device_owner_id),
    device_guid        varchar(255) not null,
    name               varchar(1024),
    email              varchar(1024),
    email_type_id      integer references email_type(email_type_id),
    session_id         varchar(4096),
    properties         varchar(4096),
    assignedToId       integer,
    assignedToName     varchar(255),
    create_time        timestamp with time zone,
    update_version     integer      not null default 0,
    time_correction    integer,
    replica_device_id  integer
);

alter table device add column change_timestamps varchar(4096);
COMMENT ON COLUMN device.change_timestamps is 'Timestamps used to track change events on the device';

COMMENT on TABLE device is 'Device - all the devices known to the system';
COMMENT ON COLUMN device.device_id is 'Unique numeric identifier for the device';
COMMENT ON COLUMN device.device_type_id is 'Device type - windows, android etc.';
COMMENT ON COLUMN device.device_owner_id is 'Device Owner - references device_owner entity';
COMMENT ON COLUMN device.device_guid is 'Global unique identifer for the device generated on the device itself';
COMMENT ON COLUMN device.name is 'Name of the device - could be machine name or some other identifier';
COMMENT ON COLUMN device.email is 'Email id that the device is registered with';
COMMENT ON COLUMN device.email_type_id is 'Email type id indicating work or personal email';
COMMENT ON COLUMN device.session_id is 'Android device registration ID to google cloud messaging';
COMMENT ON COLUMN device.properties is 'Key value pairs of device-type specific properties for the device';
COMMENT ON COLUMN device.create_time is 'Time the device was created';
COMMENT ON COLUMN device.update_version is 'Version counter used to check for concurrent updates';
COMMENT ON COLUMN device.assignedToId is 'Id of the admin assigned to this device';
COMMENT ON COLUMN device.assignedToName is 'Name of the admin assigned to this device';
COMMENT ON COLUMN device.time_correction is 'Time correction in milliseconds to apply to behavior event times from this device';
COMMENT ON COLUMN device.replica_device_id is 'Id of mapped device in test org. All events for device will be mapped to this replica device id and rerun through analytics';


--
-- Index on the device type column
--
create index idx_dev_device_type_id on device(device_type_id);

--
-- Index on the device guid column
--
create index idx_dev_device_guid on device(device_guid);

--
-- Table: device_group_device
--
CREATE TABLE device_group_device (
    device_group_id    integer not null references device_group(device_group_id),
    device_id          integer not null references device(device_id),
    primary key(device_group_id, device_id)
);

COMMENT ON TABLE device_group_device is 'Device that are part of a device group';
COMMENT ON COLUMN device_group_device.device_group_id is 'Unique identifier for the device group';
COMMENT ON COLUMN device_group_device.device_id is 'Unique identifier for the device';


--
-- Admin Role Types
--
CREATE TABLE admin_role_type (
   role_type_id   integer not null primary key,
   name           varchar(1024) not null
);

COMMENT on TABLE admin_role_type is 'Application access role type - VIEW_ONLY, ALL etc.';
COMMENT ON COLUMN admin_role_type.role_type_id is 'Unique identifier for the role type';
COMMENT ON COLUMN admin_role_type.name is 'Name of the role type';

--
-- Default values for the admin_role_type table
--
INSERT INTO admin_role_type values (1, 'ALL');
INSERT INTO admin_role_type values (2, 'VIEW_ONLY');
INSERT INTO admin_role_type values (3, 'ALL_AND_CSR');

--
-- Table: admin_registration
--
CREATE TABLE admin_registration (
   org_id              integer references organization(org_id),
   email_id            varchar(255) not null,
   domain_name         varchar(255) not null,
   phone               varchar(32),
   first_name          varchar(255),
   last_name           varchar(255),
   admin_role          integer not null references admin_role_type(role_type_id),
   org_type_id         integer not null references org_type(org_type_id),
   confirmation_code   bigint not null,
   confirmed           boolean not null,
   create_time         timestamp with time zone  not null,
   confirm_time        timestamp with time zone,
   created_by          varchar(255),
   auth_method		   varchar(255),
   primary key (email_id, confirmation_code)
);

COMMENT ON TABLE admin_registration is 'Admin user Registration information';
COMMENT ON COLUMN admin_registration.org_id is 'Organization id - will only be populated when an admin creates another admin';
COMMENT ON COLUMN admin_registration.email_id is 'Email id that will serve as the login id as well';
COMMENT ON COLUMN admin_registration.domain_name is 'Domain Name that the registration is for';
COMMENT ON COLUMN admin_registration.phone is 'Phone number';
COMMENT ON COLUMN admin_registration.first_name is 'First name';
COMMENT ON COLUMN admin_registration.last_name is 'Last name';
COMMENT ON COLUMN admin_registration.admin_role is 'Role that the admin has';
COMMENT ON COLUMN admin_registration.org_type_id is 'Organization type - Consumer or Business';
COMMENT ON COLUMN admin_registration.confirmation_code is 'Confirmation code generated during registration';
COMMENT ON COLUMN admin_registration.confirmed is 'Whether the registration has been confirmed or not';
COMMENT ON COLUMN admin_registration.create_time is 'When the registration request was created';
COMMENT ON COLUMN admin_registration.confirm_time is 'When the registration request was confirmed';
COMMENT ON COLUMN admin_registration.created_by is 'Login name of the admin who created this request or null if system generated';
COMMENT ON COLUMN admin_registration.auth_method is 'Authentication method for the admin';
--
-- Create index on org id to allow for searches for registration requests for an org
--
create index idx_org_admin_reg on admin_registration(org_id);

--
-- Table: admin_login
--
CREATE TABLE admin_login (
   login_id           serial       not null primary key,
   login_name         varchar(255) unique not null,
   password           varchar(128) ,
   admin_contact_id   bigint references contact(contact_id) not null,
   password_reset     boolean not null,
   reset_code         bigint,
   locked             boolean not null,
   lock_reason        varchar(255),
   failed_logins      smallint not null,
   create_time        timestamp with time zone  not null,
   created_by         varchar(255),
   last_login_time    timestamp with time zone,
   update_version     integer not null default 0,
   last_failed_ip     varchar(255),
   last_failed_time   timestamp with time zone,
   last_success_ip    varchar(255),
   last_5_success_ip  varchar(2048),
   ga_secure_key      varchar(255),
   ga_registered      boolean,
   auth_method	 	  varchar(255),
   new_reset_code     varchar(255)
);

COMMENT ON TABLE admin_login is 'Admin user login information';
COMMENT ON COLUMN admin_login.login_id is 'Unique identifier for the record';
COMMENT ON COLUMN admin_login.login_name is 'login name - same as the email id';
COMMENT ON COLUMN admin_login.password is 'Password used to login - stored in encrypted form';
COMMENT ON COLUMN admin_login.admin_contact_id is 'Contact information - stores user name, phone etc.';
COMMENT ON COLUMN admin_login.password_reset is 'Whether the password has been set';
COMMENT ON COLUMN admin_login.reset_code is 'Generated code to authorize the password reset attempt.';
COMMENT ON COLUMN admin_login.locked is 'Whether the account has been locked or not';
COMMENT ON COLUMN admin_login.lock_reason is 'Reason the account was locked';
COMMENT ON COLUMN admin_login.failed_logins is 'Number of consecutive failed logins';
COMMENT ON COLUMN admin_login.create_time is 'When this record was created';
COMMENT ON COLUMN admin_login.created_by is 'Login id of the admininstrator who created this login';
COMMENT ON COLUMN admin_login.last_login_time is 'Time of last successful login';
COMMENT ON COLUMN admin_login.update_version is 'Version counter used to check for concurrent updates';
COMMENT ON COLUMN admin_login.last_failed_ip is 'IP address of client for last failed login attempt';
COMMENT ON COLUMN admin_login.last_failed_time is 'Time of last failed login';
COMMENT ON COLUMN admin_login.last_success_ip is 'IP address of client for last successful login';
COMMENT ON COLUMN admin_login.last_5_success_ip is 'IP addresses of client for last 5 successful logins';
COMMENT ON COLUMN admin_login.ga_secure_key is 'Google authenticator secure key';
COMMENT ON COLUMN admin_login.ga_registered is 'WhetSUPPORT_T1her the admin registered with Google authenticator';
COMMENT ON COLUMN admin_login.auth_method is 'The authentication method the admin uses to login';
--
-- Create index for retrieval by login_name at login time
--
CREATE INDEX idx_admin_login ON admin_login(lower(login_name));

--
-- Table: org_admin -  Admins for an organization
--
create table org_admin (
    org_id             integer       not null references organization(org_id),
    login_id           integer       not null references admin_login(login_id),
    admin_role         varchar(255)  not null,
    update_version     integer not null default 0,
    primary key (org_id, login_id, admin_role)
);

COMMENT on TABLE org_admin is 'Association of a domain with an organization';
COMMENT ON COLUMN org_admin.org_id is 'Unique identifier for the organization';
COMMENT ON COLUMN org_admin.login_id is 'Unique identifier for the admin login';
COMMENT ON COLUMN org_admin.admin_role is 'Role that the admin login has for this org';
COMMENT ON COLUMN org_domain.update_version is 'Version counter used to check for concurrent updates';

--
-- Hash list type.
--
CREATE TABLE hash_list_type (
   hash_list_type_id   integer not null primary key,
   name                varchar(1024) not null
);

COMMENT on TABLE hash_list_type is 'Hash list type - indicating whether it is a black list or white list.';
COMMENT ON COLUMN hash_list_type.hash_list_type_id is 'Unique identifier for the hash list type';
COMMENT ON COLUMN hash_list_type.name is 'Name of the hash list type';

----
---- Table: hash_info storing a list of all known executable "hash"es in the system
----
--CREATE TABLE hash_info (
----   hash_id   serial not null primary key,
--   sha256_hash              char(64) not null primary key, -- used to be unique
--   md5_hash                 char(32),
--   application_name         varchar(1024),
--   file_size                bigint,
--   auth_list_name           varchar(1024),
--   auth_list_description    varchar(1024),
--   auth_list_type           integer references hash_list_type(hash_list_type_id),
--   auth_list_source         varchar(1024),
--   auth_list_last_check     timestamp with time zone,
--   create_time              timestamp with time zone not null,
--   update_version           integer      not null default 0
--);
--
--COMMENT ON TABLE hash_info is 'Executable hash information';
---- COMMENT ON COLUMN hash_info.hash_id is 'Unique numeric identifier for the hash';
--COMMENT ON COLUMN hash_info.sha256_hash is 'Unique SHA256 hash of the executable';
--COMMENT ON COLUMN hash_info.md5_hash is 'MD5 hash - optional';
--COMMENT ON COLUMN hash_info.application_name is 'Name of the application reported via File Creation event';
--COMMENT ON COLUMN hash_info.file_size is 'Size of the file in bytes reported by the File Creation event';
--COMMENT ON COLUMN hash_info.auth_list_name is 'Name of the hash as reported by the black/white listing authority';
--COMMENT ON COLUMN hash_info.auth_list_description is 'Description of the hash as reported by the black/white listing authority';
--COMMENT ON COLUMN hash_info.auth_list_type is 'Whether it is white listed or black listed or unlisted';
--COMMENT ON COLUMN hash_info.auth_list_source is 'String indicating source of the white or black list';
--COMMENT ON COLUMN hash_info.auth_list_last_check is 'Last time the hash was checked against a white or black list';
--COMMENT ON COLUMN hash_info.description is 'Description of the hash if available';
--COMMENT ON COLUMN hash_info.create_time is 'Timestamp indicating when this record was first created';
--COMMENT ON COLUMN hash_info.update_version is 'Version counter used to check for concurrent updates';

--
-- Table: org_safe_device -  Devices identified as safe by an organization.
--
create table org_safe_device (
    org_id             integer  not null references organization(org_id),
    device_id          integer  not null references device(device_id),
    description        varchar(1024),
    create_user        varchar(1024),
    create_time        timestamp with time zone not null,
    last_update        timestamp with time zone,
    update_version     integer not null default 0,
    primary key (org_id, device_id)
);

COMMENT on TABLE org_safe_device is 'Devices identified as safe for the organization';
COMMENT ON COLUMN org_safe_device.org_id is 'Unique identifier for the organization';
COMMENT ON COLUMN org_safe_device.device_id is 'Unique identifier for the hash';
COMMENT ON COLUMN org_safe_device.description is 'Description';
COMMENT ON COLUMN org_safe_device.create_user is 'User who marked this device as safe';
COMMENT ON COLUMN org_safe_device.create_time is 'Timestamp indicating when this record was first created';
COMMENT ON COLUMN org_safe_device.last_update is 'Timestamp indicating when this record was last updated';
COMMENT ON COLUMN org_safe_device.update_version is 'Version counter used to check for concurrent updates';

--
-- Index for retrieval by org_id
-- This might not be required  - as the primary key index will probably
-- cover this query
--
create index idx_org_safe_device on org_safe_device(org_id);

--
-- Default values for the hash_list_type table
--
INSERT INTO hash_list_type values (0, 'UNLISTED');
INSERT INTO hash_list_type values (1, 'BLACK_LIST');
INSERT INTO hash_list_type values (2, 'WHITE_LIST');

--
-- Table: org_hash_list -  White and black listed hashes for an organization.
--
create table org_hash_list (
    org_id             integer  not null references organization(org_id),
    sha256_hash        char(64) not null, -- Removing reference to hash_info(hash_id),
    hash_list_type     int not null references hash_list_type(hash_list_type_id),
    application_name   varchar(1024),
    description        varchar(1024),
    create_user        varchar(1024),
    create_time        timestamp with time zone not null,
    last_update        timestamp with time zone,
    update_version     integer not null default 0,
    primary key (org_id, sha256_hash)
);

COMMENT on TABLE org_hash_list is 'White and black listed hashes for an organization.';
COMMENT ON COLUMN org_hash_list.org_id is 'Unique identifier for the organization';
COMMENT ON COLUMN org_hash_list.sha256_hash is 'Sha256 identifier for the hash';
COMMENT ON COLUMN org_hash_list.hash_list_type is 'Is it a white listed or a black listed hash?';
COMMENT ON COLUMN org_hash_list.description is 'Description';
COMMENT ON COLUMN org_hash_list.create_user is 'User who marked this device as safe';
COMMENT ON COLUMN org_hash_list.create_time is 'Timestamp indicating when this record was first created';
COMMENT ON COLUMN org_hash_list.last_update is 'Timestamp indicating when this record was last updated';
COMMENT ON COLUMN org_hash_list.update_version is 'Version counter used to check for concurrent updates';

--
-- Index for retrieval by org_id
-- This might not be required  - as the primary key index will probably
-- cover this query
--
create index idx_org_hash_list_1 on org_hash_list(org_id);

----
---- Table: org_ignored_threat -  Threats to ignore (and not display on threat status page) until further notice
----
--create table org_ignored_threat (
--    org_id             integer  not null references organization(org_id),
--    device_id          integer not null references device(device_id),
--    sha256_hash        char(64) not null, -- Removing reference to hash_info(hash_id),
--    login_id           integer  not null references admin_login(login_id), -- Created by this admin
--    is_snooze          boolean  not null,
--    description        varchar(1024),
--    create_time        timestamp with time zone  not null,
--    update_version     integer not null default 0,
--    primary key (org_id, device_id, sha256_hash)
--);
--
--COMMENT on TABLE org_ignored_threat is 'List of device+hash combos that will be ignored while displaying data in the threat status page';
--COMMENT ON COLUMN org_ignored_threat.org_id is 'Unique identifier for the organization';
--COMMENT ON COLUMN org_ignored_threat.device_id is 'Unique identifier for the device';
--COMMENT ON COLUMN org_ignored_threat.sha256_hash is 'unique Sha256 identifier for the hash';
--COMMENT ON COLUMN org_ignored_threat.login_id is 'Login id of the admin who created this entry';
--COMMENT ON COLUMN org_ignored_threat.is_snooze is 'Is it a snooze OR permanant ignore - snoozes will be removed when another threat comes in';
--COMMENT ON COLUMN org_ignored_threat.description is 'Description';
--COMMENT ON COLUMN org_ignored_threat.create_time is 'Time the ignore instruction was created';
--COMMENT ON COLUMN org_ignored_threat.update_version is 'Version counter used to check for concurrent updates';

----
---- Table: device_hash_reputation storing the reputation information for each has per device
----
--CREATE TABLE device_hash_reputation (
--   device_guid            varchar(255) not null, -- Removing reference to device(device_id),
--   sha256_hash            char(64) not null,  -- Removing reference to hash_info(hash_id),
--   trust_level            integer not null check (trust_level in (0,1,2,3,4,5,6)),
--   qualifiers             varchar(32),
--   indicators             varchar(4096), -- List of indicators
--   last_seen_times        varchar(4096), -- Map of indicator and time last seen
--   first_seen_times       varchar(4096), -- Map of indicator and time first seen
--   create_time            timestamp with time zone not null,
--   last_update            timestamp with time zone,
----   global_rep_create_time timestamp with time zone,
--   update_version         integer  not null default 0,
--   primary key (device_guid, sha256_hash)
--);
--
--COMMENT ON TABLE device_hash_reputation is 'Hash Reputation info per device';
--COMMENT ON COLUMN device_hash_reputation.device_guid is 'Unique identifier for the device';
--COMMENT ON COLUMN device_hash_reputation.sha256_hash is 'Unique identifier for the hash';
--COMMENT ON COLUMN device_hash_reputation.trust_level is 'Trust level of the hash on the device';
--COMMENT ON COLUMN device_hash_reputation.qualifiers is 'Behavior qualifiers for the hash seen on the device';
--COMMENT ON COLUMN device_hash_reputation.qualifer_counts is 'Counts of behavior qualifiers seen on the device';
--COMMENT ON COLUMN device_hash_reputation.behaviors is 'Behavior indicators for the hash seen on the device - multiple behaviors map to a single  qualifier';
--COMMENT ON COLUMN device_hash_reputation.behavior_counts is 'Counts of behavior indicators seen on the device';
--COMMENT ON COLUMN device_hash_reputation.create_time is 'Timestamp indicating when this entry was created';
--COMMENT ON COLUMN device_hash_reputation.last_update is 'Timestamp indicating when this entry was last updated';
---- COMMENT ON COLUMN device_hash_reputation.global_rep_create_time is 'Timestamp indicating when the global hash reputation entry was created. Used to allow the global_hash_reputation table to be purged periodically';
--COMMENT ON COLUMN device_hash_reputation.update_version is 'Version counter used to check for concurrent updates';
--
----
---- Table: global_hash_reputation storing the reputation information for each has per device
----
--CREATE TABLE global_hash_reputation (
--   sha256_hash            char(64) not null primary key, -- Removing reference to hash_info(hash_id),
--   trust_level            integer not null check (trust_level in (0,1,2,3,4,5,6)),
--   threat_score           numeric(2,1) check (threat_score between 1 and 5),
--   num_devices            integer not null,
--   qualifiers             varchar(32),
--   qualifer_counts        varchar(4096),
--   indicators             varchar(4096),
--   indicator_counts       varchar(4096),
--   create_time            timestamp with time zone not null,
--   last_update            timestamp with time zone,
--   update_version         integer  not null default 0 --, primary key (hash_id)
--);
--
--COMMENT ON TABLE global_hash_reputation is 'Hash Reputation info per device';
--COMMENT ON COLUMN global_hash_reputation.sha256_hash is 'Unique identifier for the hash';
--COMMENT ON COLUMN global_hash_reputation.trust_level is 'Trust level of the hash on the device';
--COMMENT ON COLUMN global_hash_reputation.num_devices is 'Number of devices on which this hash has been seen';
--COMMENT ON COLUMN global_hash_reputation.qualifiers is 'Behavior qualifiers for the hash seen on the device';
--COMMENT ON COLUMN global_hash_reputation.qualifer_counts is 'Counts of behavior qualifiers seen on the device';
--COMMENT ON COLUMN global_hash_reputation.behaviors is 'Behavior indicators for the hash seen on the device - multiple behaviors map to a single  qualifier';
--COMMENT ON COLUMN global_hash_reputation.behavior_counts is 'Counts of behavior indicators seen on the device';
--COMMENT ON COLUMN global_hash_reputation.create_time is 'Timestamp indicating when this entry was created';
--COMMENT ON COLUMN global_hash_reputation.last_update is 'Timestamp indicating when this entry was last updated';
--COMMENT ON COLUMN global_hash_reputation.update_version is 'Version counter used to check for concurrent updates';


--
-- Table: BEHAVIOR_EVENT
--
CREATE TABLE behavior_event (
   event_id          bigserial not null primary key,
   message_code      integer not null,
   device_id         integer not null references device(device_id),
   process_hash      varchar(64) not null, -- Can change this later to be a reference to hash_info
   event_time        timestamp with time zone not null,
   event_info        varchar(8192) not null,
   create_time       timestamp with time zone not null,
   update_version    integer  not null default 0
);

COMMENT on TABLE behavior_event is 'Behavior Event';
COMMENT ON COLUMN behavior_event.event_id is 'Unique identifier for the event';
COMMENT ON COLUMN behavior_event.message_code is 'Message code indicating the behavior event type.';
COMMENT ON COLUMN behavior_event.device_id is 'Unique identifier for the device reporting this event';
COMMENT ON COLUMN behavior_event.process_hash is 'Process hash for the event';
COMMENT ON COLUMN behavior_event.event_time is 'Time the event occurred.';
COMMENT ON COLUMN behavior_event.event_info is 'Event notification message serialized as a string';
COMMENT ON COLUMN behavior_event.create_time is 'Timestamp indicating when this entry was created';
COMMENT ON COLUMN behavior_event.update_version is 'Version counter used to check for concurrent updates';

--
-- Index on the device_id column
--
create index idx_dev_evt_1 on behavior_event(device_id);

--
-- Table: file_upload - Table containing uploaded file data.
--
create table file_upload (
    upload_id          serial        not null primary key,
    sha256_hash        varchar(64)   not null,
    md5_hash           varchar(32)   ,
    org_id             integer       ,
    device_id          integer       ,
    device_owner       varchar(255),
    file_name          varchar(255),
    file_size          integer,
    file_contents      oid,
    create_time        timestamp with time zone  not null,
    update_version     integer not null default 0
);

COMMENT on TABLE file_upload is 'Uploaded files';
COMMENT ON COLUMN file_upload.upload_id is 'Unique identifier for the upload';
COMMENT ON COLUMN file_upload.sha256_hash is 'SHA256 hash of the upload';
COMMENT ON COLUMN file_upload.md5_hash is 'MD5 hash of the upload';
COMMENT ON COLUMN file_upload.org_id is 'Unique identifier for the organization';
COMMENT ON COLUMN file_upload.device_id is 'Unique identifier for the device that uploaded this file';
COMMENT ON COLUMN file_upload.device_owner is 'Device owner - name or email';
COMMENT ON COLUMN file_upload.file_name is 'File name';
COMMENT ON COLUMN file_upload.file_size is 'File size';
COMMENT ON COLUMN file_upload.file_contents is 'Contents of the file';
COMMENT ON COLUMN file_upload.create_time is 'Create time';
COMMENT ON COLUMN file_upload.update_version is 'Version counter used to check for concurrent updates';
